' PicBasic Pro floating point include file
'  Must compile using -ampasm switch

' Allocate storage for FP variables for RAM starting at $20
AARGB7  var     byte $20
AARGB6  var     byte $21
AARGB5  var     byte $22
AARGB4  var     byte $23
AARGB3  var     byte $24
AARGB2  var     byte $25
aint    var     word $26        ' Integer access to AARG
AARGB1  var     aint.byte0      ' $26
AARGB0  var     aint.byte1      ' $27
AEXP    var     byte $28
SIGN    var     byte $29
FPFLAGS var     byte $2A
BARGB3  var     byte $2B
BARGB2  var     byte $2C
bint    var     word $2D        ' Integer access to BARG
BARGB1  var     bint.byte0      ' $2D
BARGB0  var     bint.byte1      ' $2E
BEXP    var     byte $2F
TEMPB3  var     byte $30
TEMPB2  var     byte $31
TEMPB1  var     byte $32
TEMPB0  var     byte $33

' Define floating point flags
IOV     var     FPFLAGS.0       ' bit0 = integer overflow flag
FOV     var     FPFLAGS.1       ' bit1 = floating point overflow flag
FUN     var     FPFLAGS.2       ' bit2 = floating point underflow flag
FDZ     var     FPFLAGS.3       ' bit3 = floating point divide by zero flag
NAN     var     FPFLAGS.4       ' bit4 = not-a-number exception flag
DOM     var     FPFLAGS.5       ' bit5 = domain error exception flag
RND     var     FPFLAGS.6       ' bit6 = floating point rounding flag, 0 = truncation
                                ' 1 = unbiased rounding to nearest LSB
SAT     var     FPFLAGS.7       ' bit7 = floating point saturate flag, 0 = terminate on
                                ' exception without saturation, 1 = terminate on
                                ' exception with saturation to appropriate value

        Goto skipfp     ' Skip past floating point library

' Floating point library must load at front
' Floating point routines are available in Microchip AN575
        asm
        include "dev_fam.inc"
        include "math16.inc"
        include "fp24.a16"      ; For 24 bit fp library
    if ($ >= 800h)
        ERROR "FP library can't exceed page 0."
    endif
        endasm

' Floating point addition
fpadd:
@       goto    FPA24   ; AARG = AARG + BARG

' Floating point subtract
fpsub:
@       goto    FPS24   ; AARG = AARG - BARG

' Floating point multiply
fpmul:
@       goto    FPM24   ; AARG = AARG * BARG

' Floating point divide
fpdiv:
@       goto    FPD24   ; AARG = AARG / BARG

' Convert int in aint to float in AARG
itofa:
@       goto    FLO1624 ; 16-bit int to 24-bit float

' Convert float in AARG to int in fpa
ftoia:
@       goto    INT2416 ; 24-bit float to 16-bit int

' Convert int in bint to float in BARG
itofb:
        Gosub   swapab  ' Can't convert in BARG so swap to AARG
        Gosub   itofa   ' 16-bit int to 24-bit float in a
'       Goto    swapab  ' Swap back (Fall through)

swapab: Swap AEXP, BEXP
        Swap AARGB0, BARGB0
        Swap AARGB1, BARGB1
        Swap AARGB2, BARGB2
        Return

skipfp: RND = 1         ' Rounding on
        SAT = 1         ' Saturation on
